@@ -19,7 +19,9 @@ from __future__ import absolute_import
 
 from oslo_config import cfg
 
-ENGINE_SERVICE = 'engine'
+ENGINE_SERVICE_ID = '__engine'
+LIBRARY_SERVICE_ID = '__library'
+DS_MANAGER_SERVICE_ID = '_ds_manager'
 
 
 class APIModel(object):
@@ -28,11 +30,13 @@ class APIModel(object):
     def __init__(self, name, bus=None):
         self.name = name
         self.dse_long_timeout = cfg.CONF.dse.long_timeout
+        self.action_retry_timeout = cfg.CONF.dse.execute_action_retry_timeout
         self.bus = bus
 
     # Note(thread-safety): blocking function
     def invoke_rpc(self, caller, name, kwargs, timeout=None):
-            local = (caller is ENGINE_SERVICE and
-                     self.bus.node.service_object(ENGINE_SERVICE) is not None)
+            local = (caller is ENGINE_SERVICE_ID and
+                     self.bus.node.service_object(
+                         ENGINE_SERVICE_ID) is not None)
             return self.bus.rpc(
                 caller, name, kwargs, timeout=timeout, local=local)
