@@ -17,15 +17,16 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
-import json
 import re
 
+from oslo_serialization import jsonutils as json
 import six
 
 from congress.api import base
 from congress.api import error_codes
 from congress.api import webservice
 from congress import exception
+from congress.library_service import library_service
 
 
 class PolicyModel(base.APIModel):
@@ -35,18 +36,17 @@ class PolicyModel(base.APIModel):
     def get_items(self, params, context=None):
         """Get items in model.
 
-        Args:
-            params: A dict-like object containing parameters
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            context: Key-values providing frame of reference of request
+        :param: context: Key-values providing frame of reference of request
 
-        Returns: A dict containing at least a 'results' key whose value is
+        :returns: A dict containing at least a 'results' key whose value is
                  a list of items in the model.  Additional keys set in the
                  dict will also be rendered for the user.
         """
         try:
             # Note(thread-safety): blocking call
-            return {"results": self.invoke_rpc(base.ENGINE_SERVICE,
+            return {"results": self.invoke_rpc(base.ENGINE_SERVICE_ID,
                                                'persistent_get_policies',
                                                {})}
         except exception.CongressException as e:
@@ -54,20 +54,18 @@ class PolicyModel(base.APIModel):
 
     # Note(thread-safety): blocking function
     def get_item(self, id_, params, context=None):
-        """Retrieve item with id id_ from model.
+        """Retrieve item with id id\_ from model.
 
-        Args:
-            id_: The ID of the item to retrieve
-            params: A dict-like object containing parameters
+        :param: id\_: The ID of the item to retrieve
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            context: Key-values providing frame of reference of request
+        :param: context: Key-values providing frame of reference of request
 
-        Returns:
-             The matching item or None if id_ does not exist.
+        :returns: The matching item or None if id\_ does not exist.
         """
         try:
             # Note(thread-safety): blocking call
-            return self.invoke_rpc(base.ENGINE_SERVICE,
+            return self.invoke_rpc(base.ENGINE_SERVICE_ID,
                                    'persistent_get_policy',
                                    {'id_': id_})
         except exception.CongressException as e:
@@ -77,26 +75,62 @@ class PolicyModel(base.APIModel):
     def add_item(self, item, params, id_=None, context=None):
         """Add item to model.
 
-        Args:
-            item: The item to add to the model
-            params: A dict-like object containing parameters
+        :param: item: The item to add to the model
+        :param: params: A dict-like object containing parameters
                     from the request query string and body.
-            id_: The ID of the item, or None if an ID should be generated
-            context: Key-values providing frame of reference of request
+        :param: id\_: The ID of the item, or None if an ID should be generated
+        :param: context: Key-values providing frame of reference of request
 
-        Returns:
-             Tuple of (ID, newly_created_item)
+        :returns: Tuple of (ID, newly_created_item)
 
-        Raises:
-            KeyError: ID already exists.
-            DataModelException: Addition cannot be performed.
+        :raises KeyError: ID already exists.
+        :raises DataModelException: Addition cannot be performed.
+        :raises BadRequest: library_policy parameter and request body both
+            present
         """
+        # case 1: parameter gives library policy UUID
+        if 'library_policy' in params:
+            if item:
+                raise exception.BadRequest(
+                    'Policy creation request with `library_policy` parameter '
+                    'must not have non-empty body.')
+            try:
+                # Note(thread-safety): blocking call
+                library_policy_object = self.invoke_rpc(
+                    base.LIBRARY_SERVICE_ID,
+                    'get_policy', {'id_': params['library_policy']})
+
+                policy_metadata = self.invoke_rpc(
+                    base.ENGINE_SERVICE_ID,
+                    'persistent_create_policy_with_rules',
+                    {'policy_rules_obj': library_policy_object},
+                    timeout=self.dse_long_timeout)
+            except exception.CongressException as e:
+                raise webservice.DataModelException.create(e)
+
+            return (policy_metadata['id'], policy_metadata)
+
+        # case 2: item contains rules
+        if 'rules' in item:
+            try:
+                library_service.validate_policy_item(item)
+                # Note(thread-safety): blocking call
+                policy_metadata = self.invoke_rpc(
+                    base.ENGINE_SERVICE_ID,
+                    'persistent_create_policy_with_rules',
+                    {'policy_rules_obj': item}, timeout=self.dse_long_timeout)
+            except exception.CongressException as e:
+                raise webservice.DataModelException.create(e)
+
+            return (policy_metadata['id'], policy_metadata)
+
+        # case 3: item does not contain rules
         self._check_create_policy(id_, item)
         name = item['name']
         try:
             # Note(thread-safety): blocking call
             policy_metadata = self.invoke_rpc(
-                base.ENGINE_SERVICE, 'persistent_create_policy',
+                base.ENGINE_SERVICE_ID, 'persistent_create_policy',
                 {'name': name,
                  'abbr': item.get('abbreviation'),
                  'kind': item.get('kind'),
@@ -125,21 +159,19 @@ class PolicyModel(base.APIModel):
     def delete_item(self, id_, params, context=None):
         """Remove item from model.
 
-        Args:
-            id_: The ID or name of the item to be removed
-            params:
-            context: Key-values providing frame of reference of request
+        :param: id\_: The ID or name of the item to be removed
+        :param: params:
+        :param: context: Key-values providing frame of reference of request
 
-        Returns:
-             The removed item.
+        :returns: The removed item.
 
-        Raises:
-            KeyError: Item with specified id_ not present.
+        :raises KeyError: Item with specified id\_ not present.
         """
         # Note(thread-safety): blocking call
-        return self.invoke_rpc(base.ENGINE_SERVICE,
+        return self.invoke_rpc(base.ENGINE_SERVICE_ID,
                                'persistent_delete_policy',
-                               {'name_or_id': id_})
+                               {'name_or_id': id_},
+                               timeout=self.dse_long_timeout)
 
     def _get_boolean_param(self, key, params):
         if key not in params:
@@ -174,8 +206,8 @@ class PolicyModel(base.APIModel):
                     'action_theory': actions, 'delta': delta,
                     'trace': trace, 'as_list': True}
             # Note(thread-safety): blocking call
-            result = self.invoke_rpc(base.ENGINE_SERVICE, 'simulate', args,
-                                     timeout=self.dse_long_timeout)
+            result = self.invoke_rpc(base.ENGINE_SERVICE_ID, 'simulate',
+                                     args, timeout=self.dse_long_timeout)
         except exception.PolicyException as e:
             (num, desc) = error_codes.get('simulate_error')
             raise webservice.DataModelException(num, desc + "::" + str(e))
@@ -207,7 +239,8 @@ class PolicyModel(base.APIModel):
                     'action': action,
                     'action_args': action_args}
             # Note(thread-safety): blocking call
-            self.invoke_rpc(base.ENGINE_SERVICE, 'execute_action', args)
+            self.invoke_rpc(base.ENGINE_SERVICE_ID, 'execute_action', args,
+                            timeout=self.action_retry_timeout)
         except exception.PolicyException as e:
             (num, desc) = error_codes.get('execute_error')
             raise webservice.DataModelException(num, desc + "::" + str(e))
