@@ -17,14 +17,9 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
-from oslo_log import log as logging
-
 from congress.datalog import utility
 
 
-LOG = logging.getLogger(__name__)
-
-
 class FactSet(object):
     """FactSet
 
@@ -78,9 +73,9 @@ class FactSet(object):
             try:
                 for index in self._indicies.keys():
                     self._remove_fact_from_index(fact, index)
-            except Exception as e:
+            except Exception:
                 self._facts.add(fact)
-                raise e
+                raise
         return changed
 
     def create_index(self, columns):
