@@ -17,10 +17,13 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
+import collections
+import copy
 import datetime
 from functools import cmp_to_key
 from functools import reduce
 import hashlib
+import inspect
 import json
 import time
 
@@ -28,9 +31,8 @@ import eventlet
 from oslo_log import log as logging
 from oslo_utils import strutils
 import six
+import yaml
 
-from congress.datalog import compile
-from congress.datalog import utility
 from congress.datasources import datasource_utils as ds_utils
 from congress.db import db_ds_table_data
 from congress.dse2 import data_service
@@ -70,16 +72,18 @@ class DataSourceDriver(data_service.DataService):
     parameters describe things like the table, column names, and
     sub-translators.
 
-    HDICT parameters with example values:
-      {'translation-type': 'HDICT',
-       'table-name': 'example_table',
-       'parent-key': 'parent_key_column',
-       'id-col': 'id_col',
-       'selector-type': 'DOT_SELECTOR',
-       'field-translators': ({'fieldname': 'field1', 'col': 'col1',
-                              'translator': {'translation-type': 'VALUE'}},
-                             {'fieldname': 'field2', 'col': 'col2',
-                              'translator': {'translation-type': 'VALUE'})}
+    ::
+
+        HDICT parameters with example values:
+          {'translation-type': 'HDICT',
+           'table-name': 'example_table',
+           'parent-key': 'parent_key_column',
+           'id-col': 'id_col',
+           'selector-type': 'DOT_SELECTOR',
+           'field-translators': ({'fieldname': 'field1', 'col': 'col1',
+                                  'translator': {'translation-type': 'VALUE'}},
+                                 {'fieldname': 'field2', 'col': 'col2',
+                                  'translator': {'translation-type': 'VALUE'})}
 
       The HDICT translator reads in a python dict and translates each key in
       the dict into a column of the output table.  The fields in the table
@@ -112,7 +116,7 @@ class DataSourceDriver(data_service.DataService):
       specified with a sub-translator, that value is included as a column
       in the top-level translator's table.
 
-      Using both parent-key and id-col at the same time is redudant, so
+      Using both parent-key and id-col at the same time is redundant, so
       DataSourceDriver will reject that configuration.
 
       The example translator expects an object such as:
@@ -127,18 +131,21 @@ class DataSourceDriver(data_service.DataService):
       second table's entries derived from the primary table's row.  For
       example, if the translator is:
 
-      {'translation-type': 'HDICT',
-       'table-name': 'example_table',
-       'selector-type': 'DOT_SELECTOR',
-       'field-translators': ({'fieldname': 'field1', 'col': 'col1',
-                              'translator': {
-           'translation-type': 'LIST',
-           'table-name': 'subtable',
-           'val-col': 'c',
-           'translator': {'translation-type': 'VALUE'}},})}
+      ::
+
+          {'translation-type': 'HDICT',
+           'table-name': 'example_table',
+           'selector-type': 'DOT_SELECTOR',
+           'field-translators': ({'fieldname': 'field1', 'col': 'col1',
+                                  'translator': {
+               'translation-type': 'LIST',
+               'table-name': 'subtable',
+               'val-col': 'c',
+               'translator': {'translation-type': 'VALUE'}},})}
 
       The object {'field1': [1, 2, 3]} will translate to one tuple in
-      example_table and three tuples in subtable:
+      example_table and three tuples in subtable::
+
         example_table: (h(1, 2, 3))
         subtable: (h(1, 2, 3), 1)
                   (h(1, 2, 3), 2)
@@ -147,7 +154,7 @@ class DataSourceDriver(data_service.DataService):
         In addition, sometimes one will have data that is structured in the
         following manor (i.e a dict contained in a list within a dict):
 
-        data:
+        data::
 
             {'id': '11111',
              'things': [{'type': 1, 'location': 2}]}
@@ -157,7 +164,7 @@ class DataSourceDriver(data_service.DataService):
         explicitly, and the schema would have 3 tables. This allows you to
         use two hdicts to represent the data.
 
-        For Example:
+        For Example::
 
          thing_translator = {
             'translation-type': 'HDICT',
@@ -182,7 +189,8 @@ class DataSourceDriver(data_service.DataService):
                   'translator': thing_translator})}
 
 
-    VDICT parameters with example values:
+    VDICT parameters with example values::
+
       {'translation-type': 'VDICT',
        'table-name': 'table',
        'parent-key': 'parent_key_column',
@@ -205,7 +213,8 @@ class DataSourceDriver(data_service.DataService):
       VDICT's 'val-col' column due to an implementation choice (the id column
       is not available until after the subtranslator runs).
 
-    LIST parameters with example values:
+    LIST parameters with example values::
+
       {'translation-type': 'LIST',
        'table-name': 'table1',
        'parent-key': 'parent_key_column',
@@ -223,7 +232,8 @@ class DataSourceDriver(data_service.DataService):
       the subtranslator of a LIST may not specify a 'parent-key' because the
       LIST's table will then have no columns.
 
-   VALUE parameters with example values:
+   VALUE parameters with example values::
+
      {'translation-type': 'VALUE',
       'extract-fn': lambda x: x.['foo']}
 
@@ -261,6 +271,8 @@ class DataSourceDriver(data_service.DataService):
     IN_LIST = 'in-list'
     OBJECTS_EXTRACT_FN = 'objects-extract-fn'
     DESCRIPTION = 'desc'
+    DATA_TYPE = 'data-type'
+    NULLABLE = 'nullable'
 
     # Name of the column name and desc when using a parent key.
     PARENT_KEY_COL_NAME = 'parent_key'
@@ -277,7 +289,7 @@ class DataSourceDriver(data_service.DataService):
     LIST_PARAMS = (TRANSLATION_TYPE, TABLE_NAME, PARENT_KEY, ID_COL, VAL_COL,
                    TRANSLATOR, PARENT_COL_NAME, OBJECTS_EXTRACT_FN,
                    PARENT_KEY_DESC, VAL_COL_DESC)
-    VALUE_PARAMS = (TRANSLATION_TYPE, EXTRACT_FN)
+    VALUE_PARAMS = (TRANSLATION_TYPE, EXTRACT_FN, DATA_TYPE, NULLABLE)
     TRANSLATION_TYPE_PARAMS = (TRANSLATION_TYPE,)
     VALID_TRANSLATION_TYPES = (HDICT, VDICT, LIST, VALUE)
 
@@ -456,7 +468,7 @@ class DataSourceDriver(data_service.DataService):
             self._table_deps[translator[self.TABLE_NAME]] = related_tables
         self._validate_translator(translator, related_tables)
         self._translators.append(translator)
-        self._schema.update(self._get_schema(translator, {}))
+        self._schema.update(self._get_schema(translator, {}).schema)
 
     def get_translator(self, translator_name):
         """Get a translator.
@@ -481,12 +493,16 @@ class DataSourceDriver(data_service.DataService):
         """
         return self._translators
 
+    SCHEMA_RETURN_TUPLE = collections.namedtuple('SchemaReturnTuple',
+                                                 'schema id_type')
+
     @classmethod
-    def _get_schema_hdict(cls, translator, schema):
+    def _get_schema_hdict(cls, translator, schema, parent_key_type=None):
         tablename = translator[cls.TABLE_NAME]
         parent_key = translator.get(cls.PARENT_KEY, None)
         id_col = translator.get(cls.ID_COL, None)
         field_translators = translator[cls.FIELD_TRANSLATORS]
+        parent_col_name = None
 
         columns = []
         # columns here would be list of dictionaries.
@@ -497,25 +513,61 @@ class DataSourceDriver(data_service.DataService):
             parent_col_name = translator.get(cls.PARENT_COL_NAME,
                                              cls.PARENT_KEY_COL_NAME)
             desc = translator.get(cls.PARENT_KEY_DESC)
-            columns.append(ds_utils.add_column(parent_col_name, desc))
+            columns.append(ds_utils.add_column(
+                parent_col_name, desc, type=parent_key_type))
 
-        for field_translator in field_translators:
+        # Sort with fields lacking parent-key coming first so that the
+        # subtranslators that need a parent field will be able to get them
+        # from the fields processed first
+
+        field_translators_with_order = [
+            (index, trans) for index, trans in enumerate(field_translators)]
+        field_translators_sorted = sorted(
+            field_translators_with_order, key=cmp_to_key(
+                cls._compare_tuple_by_subtranslator))
+
+        columns_indexed = {}
+
+        def get_current_table_col_type(name):
+            if parent_col_name and parent_col_name == name:
+                return parent_key_type
+            elif name == cls._id_col_name(id_col):
+                return None  # FIXME(ekcs): return type for ID col
+            else:
+                [type] = [column_schema.get('type') for column_schema in
+                          columns_indexed.values()
+                          if column_schema.get('name') == name]
+                return type
+
+        for (index, field_translator) in field_translators_sorted:
             col = field_translator.get(
                 cls.COL, field_translator[cls.FIELDNAME])
             desc = field_translator.get(cls.DESCRIPTION)
             subtranslator = field_translator[cls.TRANSLATOR]
-            if cls.PARENT_KEY not in subtranslator:
-                columns.append(ds_utils.add_column(col, desc))
-            cls._get_schema(subtranslator, schema)
+            if cls.PARENT_KEY in subtranslator:
+                # TODO(ekcs): disallow nullable parent key
+                cls._get_schema(subtranslator, schema,
+                                parent_key_type=get_current_table_col_type(
+                                    subtranslator[cls.PARENT_KEY]))
+            else:
+                field_type = subtranslator.get(cls.DATA_TYPE)
+                nullable = subtranslator.get(cls.NULLABLE, True)
+                columns_indexed[index] = ds_utils.add_column(
+                    col, desc, field_type, nullable)
+                cls._get_schema(subtranslator, schema)
+
+        for index in range(0, len(field_translators)):
+            if index in columns_indexed:
+                columns.append(columns_indexed[index])
 
         if tablename in schema:
             raise exception.InvalidParamException(
                 "table %s already in schema" % tablename)
         schema[tablename] = tuple(columns)
-        return schema
+        return cls.SCHEMA_RETURN_TUPLE(schema, None)
 
     @classmethod
-    def _get_schema_vdict(cls, translator, schema):
+    def _get_schema_vdict(cls, translator, schema, parent_key_type=None):
         tablename = translator[cls.TABLE_NAME]
         parent_key = translator.get(cls.PARENT_KEY, None)
         id_col = translator.get(cls.ID_COL, None)
@@ -539,10 +591,10 @@ class DataSourceDriver(data_service.DataService):
             new_schema = new_schema + (value_col,)
 
         schema[tablename] = new_schema
-        return schema
+        return cls.SCHEMA_RETURN_TUPLE(schema, None)
 
     @classmethod
-    def _get_schema_list(cls, translator, schema):
+    def _get_schema_list(cls, translator, schema, parent_key_type=None):
         tablename = translator[cls.TABLE_NAME]
         parent_key = translator.get(cls.PARENT_KEY, None)
         id_col = translator.get(cls.ID_COL, None)
@@ -565,31 +617,36 @@ class DataSourceDriver(data_service.DataService):
                                  ds_utils.add_column(value_col, val_desc))
         else:
             schema[tablename] = (ds_utils.add_column(value_col, val_desc), )
-        return schema
+        return cls.SCHEMA_RETURN_TUPLE(schema, None)
 
     @classmethod
-    def _get_schema(cls, translator, schema):
-        """Returns the schema of a translator.
+    def _get_schema(cls, translator, schema, parent_key_type=None):
+        """Returns named tuple with values:
+
+        schema: the schema of a translator,
+        id_type: the data type of the id-col, or None of absent
 
         Note: this method uses the argument schema to store
         data in since this method words recursively. It might
         be worthwhile in the future to refactor this code so this
         is not required.
+
+        :param parent_key_type: passes down the column data type which the
+                translator refers to as parent-key
         """
         cls.check_translation_type(translator.keys())
         translation_type = translator[cls.TRANSLATION_TYPE]
         if translation_type == cls.HDICT:
-            cls._get_schema_hdict(translator, schema)
+            return cls._get_schema_hdict(translator, schema, parent_key_type)
         elif translation_type == cls.VDICT:
-            cls._get_schema_vdict(translator, schema)
+            return cls._get_schema_vdict(translator, schema, parent_key_type)
         elif translation_type == cls.LIST:
-            cls._get_schema_list(translator, schema)
+            return cls._get_schema_list(translator, schema, parent_key_type)
         elif translation_type == cls.VALUE:
-            pass
+            return cls.SCHEMA_RETURN_TUPLE(schema, None)
         else:
             raise AssertionError('Unexpected translator type %s' %
                                  translation_type)
-        return schema
 
     @classmethod
     def get_schema(cls):
@@ -624,12 +681,12 @@ class DataSourceDriver(data_service.DataService):
             table_state = self.state[table_id]
         except KeyError:
             m = ("tablename '%s' does not exist'" % (table_id))
-            LOG.exception(m)
+            LOG.debug(m)
             raise exception.NotFound(m)
 
         for tup in table_state:
             d = {}
-            d['data'] = tup
+            d['data'] = utils.tuple_to_congress(tup)
             results.append(d)
         return results
 
@@ -738,11 +795,31 @@ class DataSourceDriver(data_service.DataService):
         return h
 
     @classmethod
-    def _extract_value(cls, obj, extract_fn):
+    def _extract_value(cls, obj, extract_fn, data_type, nullable=True):
         # Reads a VALUE object and returns (result_rows, h)
         if extract_fn is None:
             extract_fn = lambda x: x
-        return utils.value_to_congress(extract_fn(obj))
+        value = extract_fn(obj)
+
+        # preserve type if possible; convert to str if not Hashable
+        if not isinstance(value, collections.Hashable):
+            value = str(value)
+
+        # check that data type matches if specified in translator
+        if data_type is not None and value is not None:
+            try:
+                value = data_type.marshal(value)
+            except ValueError:
+                # Note(types): Log but tolerate type error for now so that
+                # an unintentionally over-specified type does not interfere
+                # with the operation of untyped policy engines
+                LOG.exception('Type error.')
+
+        return value
+
+    @classmethod
+    def _compare_tuple_by_subtranslator(cls, x, y):
+        return cls._compare_subtranslator(x[1], y[1])
 
     @classmethod
     def _compare_subtranslator(cls, x, y):
@@ -766,8 +843,9 @@ class DataSourceDriver(data_service.DataService):
 
         if subtrans[cls.TRANSLATION_TYPE] == cls.VALUE:
             extract_fn = subtrans.get(cls.EXTRACT_FN, None)
-            converted_values = tuple([cls._extract_value(o, extract_fn)
-                                      for o in obj])
+            data_type = subtrans.get(cls.DATA_TYPE)
+            converted_values = tuple(
+                [cls._extract_value(o, extract_fn, data_type) for o in obj])
             if id_col:
                 h = cls._compute_id(id_col, obj, converted_values)
                 new_tuples = [(table, (h, v)) for v in converted_values]
@@ -831,9 +909,10 @@ class DataSourceDriver(data_service.DataService):
 
         if subtrans[cls.TRANSLATION_TYPE] == cls.VALUE:
             extract_fn = subtrans.get(cls.EXTRACT_FN, None)
-            converted_items = tuple([(utils.value_to_congress(k),
-                                      cls._extract_value(v, extract_fn))
-                                     for k, v in obj.items()])
+            data_type = subtrans.get(cls.DATA_TYPE)
+            converted_items = tuple(
+                [(k, cls._extract_value(v, extract_fn, data_type))
+                 for k, v in obj.items()])
             if id_col:
                 h = cls._compute_id(id_col, obj, converted_items)
                 new_tuples = [(table, (h,) + i) for i in converted_items]
@@ -920,9 +999,18 @@ class DataSourceDriver(data_service.DataService):
             subtranslator = field_translator[cls.TRANSLATOR]
             if subtranslator[cls.TRANSLATION_TYPE] == cls.VALUE:
                 extract_fn = subtranslator.get(cls.EXTRACT_FN)
-                v = cls._extract_value(
-                    cls._get_value(obj, field, selector), extract_fn)
-                hdict_row[col_name] = v
+                data_type = subtranslator.get(cls.DATA_TYPE)
+                nullable = subtranslator.get(cls.NULLABLE, True)
+                try:
+                    v = cls._extract_value(
+                        cls._get_value(obj, field, selector),
+                        extract_fn, data_type, nullable)
+                    hdict_row[col_name] = v
+                except TypeError as exc:
+                    arg0 = "While translating field: %s, column: %s; " \
+                           "%s" % (field, col_name, exc.args[0])
+                    exc.args = tuple([arg0]) + exc.args[1:]
+                    raise
             else:
                 assert translator[cls.TRANSLATION_TYPE] in (cls.HDICT,
                                                             cls.VDICT,
@@ -963,7 +1051,7 @@ class DataSourceDriver(data_service.DataService):
             col = fieldtranslator.get(cls.COL,
                                       fieldtranslator[cls.FIELDNAME])
             if col in hdict_row:
-                new_row.append(utils.value_to_congress(hdict_row[col]))
+                new_row.append(hdict_row[col])
 
         if id_col:
             h = cls._compute_id(id_col, obj, new_row)
@@ -1071,55 +1159,6 @@ class DataSourceDriver(data_service.DataService):
             raise exception.InvalidParamException(
                 "Param (%s) must be in translator" % cls.TRANSLATION_TYPE)
 
-    def prepush_processor(self, data, dataindex, type=None):
-        """Called before push.
-
-        Takes as input the DATA that the receiver needs and returns
-        the payload for the message.  If this is a regular publication
-        message, make the payload just the delta; otherwise, make the
-        payload the entire table.
-        """
-        # This routine basically ignores DATA and sends a delta
-        #  of the self.prior_state and self.state, for the DATAINDEX
-        #  part of the state.
-        self.log("prepush_processor: dataindex <%s> data: %s", dataindex, data)
-        # if not a regular publication, just return the original data
-        if type != 'pub':
-            self.log("prepush_processor: returned original data")
-            if type == 'sub':
-                # Always want to send initialization of []
-                if data is None:
-                    return []
-                else:
-                    return data
-            return data
-        # grab deltas
-        to_add = self.state_set_diff(self.state, self.prior_state, dataindex)
-        to_del = self.state_set_diff(self.prior_state, self.state, dataindex)
-        self.log("to_add: %s", to_add)
-        self.log("to_del: %s", to_del)
-        # create Events
-        result = []
-        for row in to_add:
-            formula = compile.Literal.create_from_table_tuple(dataindex, row)
-            event = compile.Event(formula=formula, insert=True)
-            result.append(event)
-        for row in to_del:
-            formula = compile.Literal.create_from_table_tuple(dataindex, row)
-            event = compile.Event(formula=formula, insert=False)
-            result.append(event)
-        if len(result) == 0:
-            # Policy engine expects an empty update to be an init msg
-            #  So if delta is empty, return None, which signals
-            #  the message should not be sent.
-            result = None
-            text = "None"
-        else:
-            text = utility.iterstr(result)
-        self.log("prepush_processor for <%s> returning with %s items",
-                 dataindex, text)
-        return result
-
     def request_refresh(self):
         raise NotImplementedError('request_refresh() is not implemented.')
 
@@ -1176,8 +1215,8 @@ class DataSourceDriverEndpoints(data_service.DataServiceEndPoints):
     def request_refresh(self, context, source_id):
         return self.service.request_refresh()
 
-    def request_execute(self, context, action, action_args):
-        return self.service.request_execute(context, action, action_args)
+    def request_execute(self, context, action, action_args, wait):
+        return self.service.request_execute(context, action, action_args, wait)
 
 
 class PushedDataSourceDriver(DataSourceDriver):
@@ -1221,7 +1260,7 @@ class PushedDataSourceDriver(DataSourceDriver):
             self.state[table['tablename']] = table['tabledata']
 
     # Note (thread-safety): blocking function
-    def update_entire_data(self, table_id, objs):
+    def replace_entire_table_data(self, table_id, objs):
         LOG.info('update %s table in %s datasource', table_id, self.name)
         translator = self.get_translator(table_id)
         tablename = translator['table-name']
@@ -1244,15 +1283,39 @@ class PushedDataSourceDriver(DataSourceDriver):
             else:
                 raise Exception('Push-type datasource driver does not have ID')
 
+    # Note (thread-safety): blocking function
+    def process_webhook_notification(self, payload):
+        self.prior_state = dict(self.state)
+        # call specific webhook handler of driver
+        updated_tables = self._webhook_handler(payload)
+        self.number_of_updates += 1
+        self.last_updated_time = datetime.datetime.now()
+
+        # persist in DB
+        if self.persist_data:
+            for tablename in updated_tables:
+                if self.ds_id is not None:
+                    # Note (thread-safety): blocking call
+                    db_ds_table_data.store_ds_table_data(
+                        self.ds_id, tablename, self.state[tablename])
+                else:
+                    raise Exception(
+                        'Push-type datasource driver does not have ID')
+
 
 class PushedDataSourceDriverEndpoints(data_service.DataServiceEndPoints):
     def __init__(self, service):
         super(PushedDataSourceDriverEndpoints, self).__init__(service)
 
     # Note (thread-safety): blocking function
-    def update_entire_data(self, context, table_id, source_id, objs):
+    def replace_entire_table_data(self, context, table_id, source_id, objs):
         # Note (thread-safety): blocking call
-        return self.service.update_entire_data(table_id, objs)
+        return self.service.replace_entire_table_data(table_id, objs)
+
+    # Note (thread-safety): blocking function
+    def process_webhook_notification(self, context, payload):
+        # Note (thread-safety): blocking call
+        return self.service.process_webhook_notification(payload)
 
 
 class PollingDataSourceDriver(DataSourceDriver):
@@ -1340,6 +1403,10 @@ class PollingDataSourceDriver(DataSourceDriver):
         if self.worker_greenthread is not None:
             # Note(thread-safety): blocking call
             eventlet.greenthread.kill(self.worker_greenthread)
+            try:
+                self.worker_greenthread.wait()
+            except eventlet.support.greenlets.GreenletExit:
+                pass
             self.worker_greenthread = None
             LOG.info("killed %s polling worker thread", self.name)
 
@@ -1384,7 +1451,8 @@ class PollingDataSourceDriver(DataSourceDriver):
             self.update_from_datasource()  # sets self.state
             tablenames = set(self.state.keys()) | set(self.prior_state.keys())
             for tablename in tablenames:
-                # publishing full table and using prepush_processing to send
+                # publishing full table and using differential processing in
+                #   data_service to send
                 #   only deltas.  Useful so that if policy engine subscribes
                 #   late (or dies and comes back up), DSE can automatically
                 #   send the full table.
@@ -1429,7 +1497,7 @@ class PollingDataSourceDriver(DataSourceDriver):
         is called.
 
         :param poll_time: is the amount of time (in seconds) to wait between
-        polling rounds.
+            polling rounds.
         """
         LOG.debug("start to poll from datasource %s", self.name)
         while self._running:
@@ -1466,13 +1534,9 @@ class ExecutionDriver(object):
         self.executable_methods = {}
         self.LEADER_TIMEOUT = 5
         self._leader_node_id = None
-        # TODO(dse2): remove the if-conditional when DSE1 support ended
-        #   The conditional is temporarily present to support DSE1 services
-        #   that don't have heartbeat_callbacks.
-        if hasattr(self, 'heartbeat_callbacks'):
-            # defined in DataService class
-            self.heartbeat_callbacks[
-                'check_leader'] = self._check_leader_heartbeat
+        # defined in DataService class
+        self.heartbeat_callbacks['check_leader'] = self._check_leader_heartbeat
+        self.method_structured_args = {}
 
     def _check_leader_heartbeat(self):
         """Vacate leader if heartbeat lost"""
@@ -1503,25 +1567,28 @@ class ExecutionDriver(object):
         except Exception as e:
             LOG.exception(str(e))
 
-    def add_executable_client_methods(self, client, api_prefix):
+    def add_executable_client_methods(self, client, api_prefix, exclude=None):
         """Inspect client to get supported builtin methods
 
         param client: the datasource driver client
         param api_prefix: the filter used to filter methods
         """
+        if exclude is None:
+            exclude = []
         builtin = ds_utils.inspect_methods(client, api_prefix)
         for method in builtin:
-            self.add_executable_method(method['name'], method['args'],
-                                       method['desc'])
+            if method['name'] not in exclude:
+                self.add_executable_method(method['name'], method['args'],
+                                           method['desc'])
 
     def add_executable_method(self, method_name, method_args, method_desc=""):
         """Add executable method information.
 
-        param method_name: The name of the method to add
-        param method_args: List of arguments and description of the method,
+        :param method_name: The name of the method to add
+        :param method_args: List of arguments and description of the method,
             e.g. [{'name': 'arg1', 'description': 'arg1'},
             {'name': 'arg2', 'description': 'arg2'}]
-        param method_desc: Description of the method
+        :param method_desc: Description of the method
         """
 
         if method_name not in self.executable_methods:
@@ -1543,14 +1610,56 @@ class ExecutionDriver(object):
                   action, positional_args, named_args)
         try:
             method = self._get_method(client, action)
-            # Note(thread-safety): blocking call (potentially)
-            method(*positional_args, **named_args)
         except Exception as e:
             LOG.exception(e)
             raise exception.CongressException(
                 "driver %s tries to execute %s on arguments %s but "
                 "the method isn't accepted as an executable method."
                 % (self.name, action, action_args))
+        # if some arguments are structures, load json/yaml string into struct
+        try:
+            structured_args = self.method_structured_args.get(action)
+            if structured_args is not None:
+                positional_args = copy.deepcopy(positional_args)
+                named_args = copy.deepcopy(named_args)
+                # compute which positional args to load str->struct.
+                if 'positional' not in structured_args:
+                    if inspect.ismethod(method):
+                        method_args = inspect.getargspec(method).args[1:]
+                    else:  # function or staticmethod without special 1st arg
+                        method_args = inspect.getargspec(method).args
+                    structured_positional_args = []
+                    for (index, arg) in enumerate(method_args):
+                        if arg in structured_args['named']:
+                            structured_positional_args.append(index)
+                    structured_args['positional'] = frozenset(
+                        structured_positional_args)
+                # load selected named args
+                for arg_name in named_args:
+                    if arg_name in structured_args['named']:
+                        named_args[arg_name] = yaml.load(
+                            named_args[arg_name])
+                # load selected positional args
+                for (index, arg) in enumerate(positional_args):
+                    if index in structured_args['positional']:
+                        positional_args[index] = yaml.load(arg)
+        except yaml.parser.ParserError as e:
+            LOG.exception(e)
+            raise exception.CongressException(
+                "driver %s tries to execute %s on arguments %s but "
+                "loading of JSON/YAML in designated arguments failed due to "
+                "invalid format."
+                % (self.name, action, action_args))
+
+        # Note(thread-safety): blocking call (potentially)
+        try:
+            return method(*positional_args, **named_args)
+        except Exception as e:
+            LOG.exception(e)
+            raise exception.CongressException(
+                "driver %s tries to execute %s on arguments %s but "
+                "the method raised an exception."
+                % (self.name, action, action_args))
 
     def get_actions(self):
         """Return all supported actions of a datasource driver.
@@ -1558,12 +1667,13 @@ class ExecutionDriver(object):
         Action should be a service API or a user-defined function.
         This method should return a dict for all supported actions,
         together with optional descriptions for each action and its
-        required/supported arguments. E.g.
-        {'results': [{'name': 'execute1',
-                      'args': [{"name": 'arg1', "description": "None"},
-                               {"name": 'arg2', "description": "None"}],
-                      'description': 'execute function 1'}]
-        }
+        required/supported arguments. E.g.::
+
+            {'results': [{'name': 'execute1',
+                          'args': [{"name": 'arg1', "description": "None"},
+                                   {"name": 'arg2', "description": "None"}],
+                          'description': 'execute function 1'}]
+            }
         """
         actions = []
         # order by name so that use can find out actions easily
@@ -1575,18 +1685,21 @@ class ExecutionDriver(object):
         return {'results': actions}
 
     # Note(thread-safety): blocking function
-    def request_execute(self, context, action, action_args):
+    def request_execute(self, context, action, action_args, wait):
         """Accept execution requests and execute requests from leader"""
         node_id = context.get('node_id', None)
+        th = None
         if self._leader_node_id == node_id:
-                # Note(thread-safety): blocking call
-                self.execute(action, action_args)
+            # Note(thread-safety): blocking call
+            th = eventlet.spawn(self.execute, action, action_args)
         elif node_id is not None:
             if self._leader_node_id is None:
                 self._leader_node_id = node_id
                 LOG.debug('New local leader %s selected', self._leader_node_id)
                 # Note(thread-safety): blocking call
-                self.execute(action, action_args)
+                th = eventlet.spawn(self.execute, action, action_args)
+        if wait and th:
+            th.wait()
 
     # Note(thread-safety): blocking function (in some subclasses)
     def execute(self, action, action_args):
@@ -1594,7 +1707,8 @@ class ExecutionDriver(object):
 
         Action can be a service API or a user-defined function
         :param action: a user-defined function or a service API call
-        :param action_args: in format of
+        :param action_args: in format of::
+
            {'positional': ['arg1', 'arg2'],
             'named': {'key1': 'value1', 'key2': 'value2'}}
         """
@@ -1606,7 +1720,7 @@ class ExecutionDriver(object):
     def _convert_args(self, positional_args):
         """Convert positional args to optional/named args.
 
-        :param <list> positional_args: items are assumed being
+        :param: <list> positional_args: items are assumed being
         ordered as ['key1', 'value1', 'key2', 'value2',].
         :return <dict>: {'key1': 'value1', 'key2': 'value2'}
         """
