@@ -19,23 +19,26 @@ from __future__ import absolute_import
 import functools
 import inspect
 import re
-from six.moves.urllib import parse as urlparse
 
-import keystoneauth1.identity.v2 as v2
-import keystoneauth1.identity.v3 as v3
-import keystoneauth1.session as kssession
+from keystoneauth1 import loading as kaloading
 
 from congress.datasources import constants
 
 
+def typed_value_trans(type):
+    return {'translation-type': 'VALUE', 'data-type': type}
+
+
 def get_openstack_required_config():
     return {'auth_url': constants.REQUIRED,
             'endpoint': constants.OPTIONAL,
             'region': constants.OPTIONAL,
             'username': constants.REQUIRED,
             'password': constants.REQUIRED,
-            'tenant_name': constants.REQUIRED,
-            'project_name': constants.OPTIONAL,
+            'user_domain_name': constants.OPTIONAL,
+            'project_domain_name': constants.OPTIONAL,
+            'tenant_name': constants.OPTIONAL,
+            'project_name': constants.REQUIRED,
             'poll_time': constants.OPTIONAL}
 
 
@@ -64,9 +67,14 @@ def update_state_on_changed(root_table_name):
     return outer
 
 
-def add_column(colname, desc=None):
+def add_column(colname, desc=None, type=None, nullable=True):
     """Adds column in the form of dict."""
-    return {'name': colname, 'desc': desc}
+    col_dict = {'name': colname, 'desc': desc}
+    if type is not None:
+        col_dict['type'] = str(type)
+    if not nullable:
+        col_dict['nullable'] = False
+    return col_dict
 
 
 def inspect_methods(client, api_prefix):
@@ -153,27 +161,19 @@ def inspect_methods(client, api_prefix):
 
 # Note (thread-safety): blocking function
 def get_keystone_session(creds):
-    url_parts = urlparse.urlparse(creds['auth_url'])
-    path = url_parts.path.lower()
-    if path.startswith('/v3'):
-        # Use v3 plugin to authenticate
-        # Note (thread-safety): blocking call
-        auth = v3.Password(
-            auth_url=creds['auth_url'],
-            username=creds['username'],
-            password=creds['password'],
-            project_name=creds.get('project_name') or creds.get('tenant_name'),
-            user_domain_name=creds.get('user_domain_name', 'default'),
-            project_domain_name=creds.get('project_domain_name', 'default'))
 
-    else:
-        # Use v2 plugin
-        # Note (thread-safety): blocking call
-        auth = v2.Password(auth_url=creds['auth_url'],
-                           username=creds['username'],
-                           password=creds['password'],
-                           tenant_name=creds['tenant_name'])
-
-    # Note (thread-safety): blocking call?
-    session = kssession.Session(auth=auth)
+    auth_details = {}
+    auth_details['auth_url'] = creds['auth_url']
+    auth_details['username'] = creds['username']
+    auth_details['password'] = creds['password']
+    auth_details['project_name'] = (creds.get('project_name') or
+                                    creds.get('tenant_name'))
+    auth_details['tenant_name'] = creds.get('tenant_name')
+    auth_details['user_domain_name'] = creds.get('user_domain_name', 'Default')
+    auth_details['project_domain_name'] = creds.get('project_domain_name',
+                                                    'Default')
+    loader = kaloading.get_plugin_loader('password')
+    auth_plugin = loader.load_from_options(**auth_details)
+    session = kaloading.session.Session().load_from_options(
+        auth=auth_plugin)
     return session
