@@ -13,6 +13,22 @@
 #    under the License.
 #
 
+"""Schema change history
+
+date: 2018-03-15
+changes:
+
+ - (incompatible) Removed the `hosts` table for OS hosts information because
+   access to the information has been removed from the latest Nova API and
+   client.
+ - Added the `hypervisors` table for hypervisor information.
+
+date: 2017-10-01
+changes:
+
+ - Added the `tags` table for server tags information.
+"""
+
 from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
@@ -21,10 +37,12 @@ import novaclient.client
 from oslo_log import log as logging
 import six
 
+from congress import data_types
 from congress.datasources import constants
 from congress.datasources import datasource_driver
 from congress.datasources import datasource_utils as ds_utils
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -32,13 +50,14 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
                  datasource_driver.ExecutionDriver):
     SERVERS = "servers"
     FLAVORS = "flavors"
-    HOSTS = "hosts"
-    FLOATING_IPS = "floating_IPs"
+    HYPERVISORS = "hypervisors"
     SERVICES = 'services'
     AVAILABILITY_ZONES = "availability_zones"
+    TAGS = "tags"
 
-    # This is the most common per-value translator, so define it once here.
-    value_trans = {'translation-type': 'VALUE'}
+    value_trans_str = ds_utils.typed_value_trans(data_types.Str)
+    value_trans_bool = ds_utils.typed_value_trans(data_types.Bool)
+    value_trans_int = ds_utils.typed_value_trans(data_types.Int)
 
     def safe_id(x):
         if isinstance(x, six.string_types):
@@ -54,33 +73,42 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         'selector-type': 'DOT_SELECTOR',
         'field-translators':
             ({'fieldname': 'id', 'desc': 'The UUID for the server',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'name', 'desc': 'Name of the server',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'hostId', 'col': 'host_id',
-              'desc': 'The UUID for the host', 'translator': value_trans},
+              'desc': 'The UUID for the host', 'translator': value_trans_str},
              {'fieldname': 'status', 'desc': 'The server status',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'tenant_id', 'desc': 'The tenant ID',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'user_id',
               'desc': 'The user ID of the user who owns the server',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'image', 'col': 'image_id',
               'desc': 'Name or ID of image',
               'translator': {'translation-type': 'VALUE',
                              'extract-fn': safe_id}},
              {'fieldname': 'flavor', 'col': 'flavor_id',
-              'desc': 'Name of the flavor',
+              'desc': 'ID of the flavor',
               'translator': {'translation-type': 'VALUE',
                              'extract-fn': safe_id}},
              {'fieldname': 'OS-EXT-AZ:availability_zone', 'col': 'zone',
               'desc': 'The availability zone of host',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'OS-EXT-SRV-ATTR:hypervisor_hostname',
-              'desc': ('The hostname of hypervisor where the server is' +
+              'desc': ('The hostname of hypervisor where the server is '
                        'running'),
-              'col': 'host_name', 'translator': value_trans})}
+              'col': 'host_name', 'translator': value_trans_str},
+             {'fieldname': 'tags',
+              'translator': {'translation-type': 'LIST',
+                             'table-name': TAGS,
+                             'parent-key': 'id',
+                             'parent-col-name': 'server_id',
+                             'parent-key-desc': 'UUID of server',
+                             'val-col': 'tag',
+                             'val-col-desc': 'server tag string',
+                             'translator': value_trans_str}})}
 
     flavors_translator = {
         'translation-type': 'HDICT',
@@ -88,47 +116,34 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         'selector-type': 'DOT_SELECTOR',
         'field-translators':
             ({'fieldname': 'id', 'desc': 'ID of the flavor',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'name', 'desc': 'Name of the flavor',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'vcpus', 'desc': 'Number of vcpus',
-              'translator': value_trans},
+              'translator': value_trans_int},
              {'fieldname': 'ram', 'desc': 'Memory size in MB',
-              'translator': value_trans},
+              'translator': value_trans_int},
              {'fieldname': 'disk', 'desc': 'Disk size in GB',
-              'translator': value_trans},
+              'translator': value_trans_int},
              {'fieldname': 'ephemeral', 'desc': 'Ephemeral space size in GB',
-              'translator': value_trans},
+              'translator': value_trans_int},
              {'fieldname': 'rxtx_factor', 'desc': 'RX/TX factor',
-              'translator': value_trans})}
+              'translator': ds_utils.typed_value_trans(data_types.Float)})}
 
-    hosts_translator = {
+    hypervisors_translator = {
         'translation-type': 'HDICT',
-        'table-name': HOSTS,
+        'table-name': HYPERVISORS,
         'selector-type': 'DOT_SELECTOR',
         'field-translators':
-            ({'fieldname': 'host_name', 'desc': 'Name of host',
-              'translator': value_trans},
-             {'fieldname': 'service', 'desc': 'Enabled service',
-              'translator': value_trans},
-             {'fieldname': 'zone', 'desc': 'The availability zone of host',
-              'translator': value_trans})}
-
-    floating_ips_translator = {
-        'translation-type': 'HDICT',
-        'table-name': FLOATING_IPS,
-        'selector-type': 'DOT_SELECTOR',
-        'field-translators':
-            ({'fieldname': 'fixed_ip', 'desc': 'Fixed IP Address',
-              'translator': value_trans},
-             {'fieldname': 'id', 'desc': 'Unique ID',
-              'translator': value_trans},
-             {'fieldname': 'ip', 'desc': 'IP Address',
-              'translator': value_trans},
-             {'fieldname': 'instance_id',
-              'desc': 'Name or ID of host', 'translator': value_trans},
-             {'fieldname': 'pool', 'desc': 'Name of Floating IP Pool',
-              'translator': value_trans})}
+            ({'fieldname': 'hypervisor_hostname', 'desc': 'Hypervisor host',
+              'translator': value_trans_str},
+             {'fieldname': 'id', 'desc': 'hypervisori id',
+              # untyped: depends on api microversion
+              'translator': {'translation-type': 'VALUE'}},
+             {'fieldname': 'state', 'desc': 'State of the hypervisor',
+              'translator': value_trans_str},
+             {'fieldname': 'status', 'desc': 'Status of the hypervisor',
+              'translator': value_trans_str})}
 
     services_translator = {
         'translation-type': 'HDICT',
@@ -136,21 +151,21 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         'selector-type': 'DOT_SELECTOR',
         'field-translators':
             ({'fieldname': 'id', 'col': 'service_id', 'desc': 'Service ID',
-              'translator': value_trans},
+              'translator': value_trans_int},
              {'fieldname': 'binary', 'desc': 'Service binary',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'host', 'desc': 'Host Name',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'zone', 'desc': 'Availability Zone',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'status', 'desc': 'Status of service',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'state', 'desc': 'State of service',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'updated_at', 'desc': 'Last updated time',
-              'translator': value_trans},
+              'translator': value_trans_str},
              {'fieldname': 'disabled_reason', 'desc': 'Disabled reason',
-              'translator': value_trans})}
+              'translator': value_trans_str})}
 
     availability_zones_translator = {
         'translation-type': 'HDICT',
@@ -158,14 +173,13 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         'selector-type': 'DOT_SELECTOR',
         'field-translators':
             ({'fieldname': 'zoneName', 'col': 'zone',
-              'desc': 'Availability zone name', 'translator': value_trans},
+              'desc': 'Availability zone name', 'translator': value_trans_str},
              {'fieldname': 'zoneState', 'col': 'state',
               'desc': 'Availability zone state',
-              'translator': value_trans})}
+              'translator': value_trans_str})}
 
-    TRANSLATORS = [servers_translator, flavors_translator, hosts_translator,
-                   floating_ips_translator, services_translator,
-                   availability_zones_translator]
+    TRANSLATORS = [servers_translator, flavors_translator, services_translator,
+                   hypervisors_translator, availability_zones_translator]
 
     def __init__(self, name='', args=None):
         super(NovaDriver, self).__init__(name, args)
@@ -173,13 +187,18 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         self.creds = args
         session = ds_utils.get_keystone_session(self.creds)
         self.nova_client = novaclient.client.Client(
-            version=self.creds.get('api_version', '2'), session=session)
+            version=self.creds.get('api_version', '2.26'), session=session)
         self.add_executable_method('servers_set_meta',
                                    [{'name': 'server',
                                     'description': 'server id'},
-                                    {'name': 'meta',
-                                     'description': 'metadata pairs, ' +
-                                     'e.g. meta1=val1 meta2=val2'}],
+                                    {'name': 'meta-key1',
+                                     'description': 'meta key 1'},
+                                    {'name': 'meta-value1',
+                                     'description': 'value for meta key1'},
+                                    {'name': 'meta-keyN',
+                                     'description': 'meta key N'},
+                                    {'name': 'meta-valueN',
+                                     'description': 'value for meta keyN'}],
                                    "A wrapper for servers.set_meta()")
         self.add_executable_client_methods(self.nova_client, 'novaclient.v2.')
         self.initialize_update_methods()
@@ -207,14 +226,10 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
             self.nova_client.flavors.list())
         self.add_update_method(flavors_method, self.flavors_translator)
 
-        hosts_method = lambda: self._translate_hosts(
-            self.nova_client.hosts.list())
-        self.add_update_method(hosts_method, self.hosts_translator)
-
-        floating_ips_method = lambda: self._translate_floating_ips(
-            self.nova_client.floating_ips.list())
-        self.add_update_method(floating_ips_method,
-                               self.floating_ips_translator)
+        hypervisors_method = lambda: self._translate_hypervisors(
+            self.nova_client.hypervisors.list())
+        self.add_update_method(hypervisors_method,
+                               self.hypervisors_translator)
 
         services_method = lambda: self._translate_services(
             self.nova_client.services.list())
@@ -234,15 +249,11 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         row_data = NovaDriver.convert_objs(obj, NovaDriver.flavors_translator)
         return row_data
 
-    @ds_utils.update_state_on_changed(HOSTS)
-    def _translate_hosts(self, obj):
-        row_data = NovaDriver.convert_objs(obj, NovaDriver.hosts_translator)
-        return row_data
-
-    @ds_utils.update_state_on_changed(FLOATING_IPS)
-    def _translate_floating_ips(self, obj):
-        row_data = NovaDriver.convert_objs(obj,
-                                           NovaDriver.floating_ips_translator)
+    @ds_utils.update_state_on_changed(HYPERVISORS)
+    def _translate_hypervisors(self, obj):
+        row_data = NovaDriver.convert_objs(
+            obj,
+            NovaDriver.hypervisors_translator)
         return row_data
 
     @ds_utils.update_state_on_changed(SERVICES)
@@ -273,13 +284,15 @@ class NovaDriver(datasource_driver.PollingDataSourceDriver,
         'execute[p(x)]' doesn't take optional args at the moment.
         Therefore, this function translates the positional ARGS
         to optional args and call the servers.set_meta() api.
-        :param <list> args: expected server ID and pairs of meta
-        data in positional args such as:
-        {'positional': ['server_id', 'meta1', 'value1', 'meta2', 'value2']}
+        :param: <list> args: expected server ID and pairs of meta
+        data in positional args such as::
 
-        Usage:
-        execute[nova.servers_set_meta(svr_id, meta1, val1, meta2, val2) :-
-            triggering_table(id)
+            {'positional': ['server_id', 'meta1', 'value1', 'meta2', 'value2']}
+
+        Usage::
+
+            execute[nova.servers_set_meta(svr_id, meta1, val1, meta2, val2) :-
+                triggering_table(id)
         """
         action = 'servers.set_meta'
         positional_args = args.get('positional', [])
