@@ -39,17 +39,10 @@ import requests
 
 from congress.datasources import constants
 from congress.datasources import datasource_driver
-from congress.managers import datasource as datasource_mgr
 
 LOG = logging.getLogger(__name__)
 
 
-def d6service(name, keys, inbox, datapath, args):
-    """This method is called by d6cage to create a dataservice instance."""
-
-    return PlexxiDriver(name, keys, inbox, datapath, args)
-
-
 class PlexxiDriver(datasource_driver.PollingDataSourceDriver,
                    datasource_driver.ExecutionDriver):
     HOSTS = "hosts"
@@ -66,9 +59,8 @@ class PlexxiDriver(datasource_driver.PollingDataSourceDriver,
     PORTS = "ports"
     NETWORKLINKS = "networklinks"
 
-    def __init__(self, name='', keys='', inbox=None, datapath=None, args=None,
-                 session=None):
-        super(PlexxiDriver, self).__init__(name, keys, inbox, datapath, args)
+    def __init__(self, name='', args=None, session=None):
+        super(PlexxiDriver, self).__init__(name, args=args)
         datasource_driver.ExecutionDriver.__init__(self)
         self.exchange = session
         self.creds = args
@@ -577,7 +569,7 @@ class PlexxiDriver(datasource_driver.PollingDataSourceDriver,
         VMs that have the same name in the Plexxi table and the Nova Table.
         """
         try:
-            datasources = datasource_mgr.DataSourceManager.get_datasources()
+            datasources = self.node.get_datasources()
             for datasource in datasources:
                 if datasource['driver'] == 'nova':
                     repeated_name_rule = ('{"rule": "RepeatedName' +
@@ -639,3 +631,17 @@ class PlexxiDriver(datasource_driver.PollingDataSourceDriver,
                             LOG.info(desc + " in PlexxiCore.")
         except Exception:
             LOG.exception("error in name_response")
+
+    def execute(self, action, action_args):
+        """Overwrite ExecutionDriver.execute()."""
+        # action can be written as a method or an API call.
+        func = getattr(self, action, None)
+        if func and self.is_executable(func):
+            func(action_args)
+        # TODO(aimeeu)
+        # The 'else' block is where we would execute native Plexxi actions
+        # (actions implemented in the Plexxi libraries). However, that is
+        # hard to do because of the rest of the way the driver is written.
+        # The question for the 'else' block is whether it's worth exposing
+        # all the native Plexxi actions. See comments in review
+        # https://review.openstack.org/#/c/335539/
