@@ -61,9 +61,9 @@ class CongressException(Exception):
         """FIXME(thinrichs):
 
         We just want name and data as fields.
-        :param name will be a name from error_codes, which includes the basic
+        :param: name will be a name from error_codes, which includes the basic
         message.
-        :param data will contain specifics for this instance of the
+        :param: data will contain specifics for this instance of the
         exception, e.g. a description error message.
         """
         self.data = kwargs.get('data', None)
@@ -132,6 +132,11 @@ class NotFound(CongressException):
     code = 404
 
 
+class Unavailable(CongressException):
+    msg_fmt = _("Service Unavailable.")
+    code = 503
+
+
 class PolicyNotAuthorized(Forbidden):
     msg_fmt = _("Policy doesn't allow %(action)s to be performed.")
 
@@ -160,6 +165,10 @@ class LazyTable(BadRequest):
     msg_fmt = _("table %(lazy_table)s is a lazy table and is not subscribed.")
 
 
+class InvalidPolicyInput(BadRequest):
+    msg_fmt = _('Input policy item violates schema.')
+
+
 # NOTE(thinrichs): The following represent different kinds of
 #   exceptions: the policy compiler and the policy runtime, respectively.
 class PolicyException(CongressException):
@@ -180,6 +189,15 @@ class PolicyRuntimeException(CongressException):
     pass
 
 
+class DatabaseError(CongressException):
+    msg_fmt = _("Database backend experienced an unknown error.")
+
+
+class DatabaseDataError(BadRequest):
+    msg_fmt = _("Database backend experienced a data error. "
+                "E.g. value out of range, text string too long.")
+
+
 class IncompleteSchemaException(CongressException):
     pass
 
@@ -212,12 +230,18 @@ class DatasourceNameInUse(Conflict):
     msg_fmt = _("Datasource already in use with name %(value)s")
 
 
+class InvalidDatasourceName(BadConfig):
+    msg_fmt = _("Datasource name %(value) is invalid. Cannot be empty or "
+                "start with underscore.  Must be valid in policy language")
+
+
 class DatasourceNotFound(NotFound):
     msg_fmt = _("Datasource not found %(id)s")
 
 
 class DriverNotFound(NotFound):
-    msg_fmt = _("Driver not found %(id)s")
+    msg_fmt = _("Driver not found %(id)s. Either not supported or disabled by "
+                "deployer in Congress config")
 
 
 class DatasourceCreationError(BadConfig):
