@@ -17,19 +17,16 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
-import copy
-import os
-import os.path
-import re
 import sys
 
 from oslo_config import cfg
-
 from oslo_log import log as logging
 
 from congress.api import action_model
 from congress.api import application
+from congress.api import base as api_base
 from congress.api import datasource_model
+from congress.api import library_policy_model
 from congress.api import policy_model
 from congress.api import router
 from congress.api import row_model
@@ -38,14 +35,15 @@ from congress.api import schema_model
 from congress.api import status_model
 from congress.api.system import driver_model
 from congress.api import table_model
+from congress.api import webhook_model
 from congress.db import datasources as db_datasources
+from congress.dse2 import datasource_manager as ds_manager
 from congress.dse2 import dse_node
 from congress import exception
+from congress.library_service import library_service
 from congress.policy_engines import agnostic
 
-
 LOG = logging.getLogger(__name__)
-ENGINE_SERVICE_NAME = 'engine'
 
 
 def create2(node_id=None, bus_id=None, existing_node=None,
@@ -55,13 +53,13 @@ def create2(node_id=None, bus_id=None, existing_node=None,
     Creates a DseNode if one is not provided and adds policy_engine,
     datasources, api to that node.
 
-    :param node_id is node_id of DseNode to be created
-    :param bus_id is partition_id of DseNode to be created
-    :param existing_node is a DseNode (optional; in lieu of previous 2 params)
-    :param policy_engine controls whether policy_engine is included
-    :param datasources controls whether datasources are included
-    :param api controls whether API is included
-    :returns DseNode
+    :param: node_id is node_id of DseNode to be created
+    :param: bus_id is partition_id of DseNode to be created
+    :param: existing_node is a DseNode (optional; in lieu of previous 2 params)
+    :param: policy_engine controls whether policy_engine is included
+    :param: datasources controls whether datasources are included
+    :param: api controls whether API is included
+    :returns: DseNode
     """
     # create DseNode if existing_node not given
     if existing_node is None:
@@ -75,40 +73,39 @@ def create2(node_id=None, bus_id=None, existing_node=None,
 
     # create services as required
     services = {}
-    if api:
-        LOG.info("Registering congress API service on node %s", node.node_id)
-        services['api'], services['api_service'] = create_api()
-        node.register_service(services['api_service'])
 
-    if policy_engine:
-        LOG.info("Registering congress PolicyEngine service on node %s",
-                 node.node_id)
-        services[ENGINE_SERVICE_NAME] = create_policy_engine()
-        node.register_service(services[ENGINE_SERVICE_NAME])
-        initialize_policy_engine(services[ENGINE_SERVICE_NAME])
+    # Load all configured drivers
+    dse_node.DseNode.load_drivers()
 
     if datasources:
         LOG.info("Registering congress datasource services on node %s",
                  node.node_id)
         services['datasources'] = create_datasources(node)
+        services['ds_manager'] = ds_manager.DSManagerService(
+            api_base.DS_MANAGER_SERVICE_ID)
+        node.register_service(services['ds_manager'])
 
-        # datasource policies would be created by respective PE's synchronizer
-        # for ds in services['datasources']:
-        #    try:
-        #        utils.create_datasource_policy(ds, ds.name,
-        #                                       ENGINE_SERVICE_NAME)
-        #    except (exception.BadConfig,
-        #            exception.DatasourceNameInUse,
-        #            exception.DriverNotFound,
-        #            exception.DatasourceCreationError) as e:
-        #        LOG.exception("Datasource %s creation failed. %s" % (ds, e))
-        #        node.unregister_service(ds)
-
-    # start synchronizer and other periodic tasks
     if policy_engine:
-        services[ENGINE_SERVICE_NAME].start_policy_synchronizer()
-    if datasources:
-        node.start_periodic_tasks()
+        LOG.info("Registering congress PolicyEngine service on node %s",
+                 node.node_id)
+        engine = create_policy_engine()
+        services[api_base.ENGINE_SERVICE_ID] = engine
+        node.register_service(engine)
+        initialize_policy_engine(engine)
+
+        # NOTE(ekcs): library service does not depend on policy engine,
+        # it is placed on the same nodes as policy engine for convenience only
+        LOG.info("Registering congress policy library service on node %s",
+                 node.node_id)
+        library = create_policy_library_service()
+        services[api_base.LIBRARY_SERVICE_ID] = library
+        node.register_service(library)
+
+    if api:
+        LOG.info("Registering congress API service on node %s", node.node_id)
+        services['api'], services['api_service'] = create_api()
+        node.register_service(services['api_service'])
+
     return services
 
 
@@ -124,6 +121,8 @@ def create_api():
 def create_api_models(bus):
     """Create all the API models and return as a dictionary for DSE2."""
     res = {}
+    res['api-library-policy'] = library_policy_model.LibraryPolicyModel(
+        'api-library-policy', bus=bus)
     res['api-policy'] = policy_model.PolicyModel('api-policy', bus=bus)
     res['api-rule'] = rule_model.RuleModel('api-rule', bus=bus)
     res['api-row'] = row_model.RowModel('api-row', bus=bus)
@@ -131,6 +130,7 @@ def create_api_models(bus):
         'api-datasource', bus=bus)
     res['api-schema'] = schema_model.SchemaModel('api-schema', bus=bus)
     res['api-table'] = table_model.TableModel('api-table', bus=bus)
+    res['api-webhook'] = webhook_model.WebhookModel('api-webhook', bus=bus)
     res['api-status'] = status_model.StatusModel('api-status', bus=bus)
     res['api-action'] = action_model.ActionsModel('api-action', bus=bus)
     res['api-system'] = driver_model.DatasourceDriverModel(
@@ -140,7 +140,7 @@ def create_api_models(bus):
 
 def create_policy_engine():
     """Create policy engine and initialize it using the api models."""
-    engine = agnostic.DseRuntime(ENGINE_SERVICE_NAME)
+    engine = agnostic.DseRuntime(api_base.ENGINE_SERVICE_ID)
     engine.debug_mode()  # should take this out for production
     return engine
 
@@ -153,6 +153,15 @@ def initialize_policy_engine(engine):
     engine.persistent_load_rules()
 
 
+def create_policy_library_service():
+    """Create policy library service."""
+    library = library_service.LibraryService(api_base.LIBRARY_SERVICE_ID)
+    # load library policies from file if none present in DB
+    if len(library.get_policies(include_rules=False)) == 0:
+        library.load_policies_from_files()
+    return library
+
+
 def create_datasources(bus):
     """Create and register datasource services ."""
     if cfg.CONF.delete_missing_driver_datasources:
@@ -175,38 +184,6 @@ def create_datasources(bus):
                           "clean up stale datasources in DB.")
             sys.exit(1)
         except Exception:
-            LOG.exception("datasource %s creation failed.", ds.name)
-            raise
-
+            LOG.exception("datasource %s creation failed. %s service may not "
+                          "be running.", ds.name, ds.driver)
     return services
-
-
-def load_data_service(service_name, config, cage, rootdir, id_):
-    """Load service.
-
-    Load a service if not already loaded. Also loads its
-    module if the module is not already loaded.  Returns None.
-    SERVICE_NAME: name of service
-    CONFIG: dictionary of configuration values
-    CAGE: instance to load service into
-    ROOTDIR: dir for start of module paths
-    ID: UUID of the service.
-    """
-    config = copy.copy(config)
-    if service_name in cage.services:
-        return
-    if 'module' not in config:
-        raise exception.DataSourceConfigException(
-            "Service %s config missing 'module' entry" % service_name)
-    module_path = config['module']
-    module_name = re.sub('[^a-zA-Z0-9_]', '_', module_path)
-    if not os.path.isabs(module_path) and rootdir is not None:
-        module_path = os.path.join(rootdir, module_path)
-    if module_name not in sys.modules:
-        LOG.info("Trying to create module %s from %s",
-                 module_name, module_path)
-        cage.loadModule(module_name, module_path)
-    LOG.info("Trying to create service %s with module %s",
-             service_name, module_name)
-    cage.createservice(name=service_name, moduleName=module_name,
-                       args=config, type_='datasource_driver', id_=id_)
