@@ -33,5 +33,5 @@ def list_opts():
              congress.utils.utils_opts,
              congress.exception.exc_log_opts,
          )),
-        ('dse', congress.dse2.dse_node._dse_opts)
+        ('dse', congress.common.config.dse_opts)
     ]
