@@ -17,9 +17,12 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
+from oslo_config import cfg
+
 from congress.api import webservice
 from congress.tests.api import base as api_base
 from congress.tests import base
+from congress.tests import helper
 
 
 class TestDriverModel(base.SqlTestCase):
@@ -27,33 +30,36 @@ class TestDriverModel(base.SqlTestCase):
         super(TestDriverModel, self).setUp()
         services = api_base.setup_config()
         self.node = services['node']
-
-        self.node.add_datasource(self._get_datasource_request())
         self.driver_model = services['api']['api-system']
 
-    def _get_datasource_request(self):
-        req = {'driver': 'fake_datasource',
-               'name': 'fake_datasource'}
-        req['config'] = {'auth_url': 'foo',
-                         'username': 'foo',
-                         'password': 'password',
-                         'tenant_name': 'foo'}
-        return req
-
     def tearDown(self):
         super(TestDriverModel, self).tearDown()
 
     def test_drivers_list(self):
         context = {}
-        expected_ret = {"results": [
-            {
-                "description": "This is a fake driver used for testing",
-                "id": "fake_datasource"
-            }
-        ]}
+        drivers = helper.supported_drivers()
+        expected_ret = sorted(drivers, key=lambda d: d['id'])
+        ret = self.driver_model.get_items({}, context)['results']
+        actual_ret = sorted(ret, key=lambda d: d['id'])
+        self.assertEqual(expected_ret, actual_ret)
 
-        ret = self.driver_model.get_items({}, context)
-        self.assertEqual(expected_ret, ret)
+    def test_drivers_list_with_disabled_drivers(self):
+        cfg.CONF.set_override('disabled_drivers', 'plexxi')
+        services = api_base.setup_config(node_id='test-node-1')
+        driver_api = services['api']['api-system']
+        drivers = [d['id'] for d in helper.supported_drivers()]
+        drivers.remove('plexxi')
+        ret = [d['id'] for d in driver_api.get_items({}, {})['results']]
+        self.assertEqual(sorted(drivers), sorted(ret))
+
+    def test_drivers_list_with_custom_drivers(self):
+        cfg.CONF.set_override(
+            'custom_driver_endpoints',
+            'test=congress.tests.test_custom_driver:TestCustomDriver')
+        services = api_base.setup_config(node_id='test-node-2')
+        driver_api = services['api']['api-system']
+        ret = [d['id'] for d in driver_api.get_items({}, {})['results']]
+        self.assertIn('test', ret)
 
     def test_driver_details(self):
         context = {
@@ -66,13 +72,14 @@ class TestDriverModel(base.SqlTestCase):
                 "password": "required",
                 "poll_time": "(optional)",
                 "region": "(optional)",
-                "project_name": "(optional)",
-                "tenant_name": "required",
+                'project_domain_name': '(optional)',
+                'user_domain_name': '(optional)',
+                "project_name": "required",
+                "tenant_name": "(optional)",
                 "username": "required"
             },
             "description": "This is a fake driver used for testing",
             "id": "fake_datasource",
-            "module": "congress.tests.fake_datasource.FakeDataSource",
             "secret": ["password"],
             "tables": [{'columns': [
                 {'description': None, 'name': 'id'},
