@@ -24,6 +24,7 @@ import eventlet
 import mock
 from oslo_utils import uuidutils
 
+from congress import data_types
 from congress.datasources import datasource_driver
 from congress.datasources import datasource_utils
 from congress.db import db_ds_table_data
@@ -72,6 +73,16 @@ class TestDatasourceDriver(base.TestCase):
             self.assertEqual(params, expected)
 
     def test_in_list_results_hdict_hdict(self):
+        class Type1(object):
+            @classmethod
+            def marshal(cls, value):
+                return value
+
+        class Type2(object):
+            @classmethod
+            def marshal(cls, value):
+                return value
+
         ports_fixed_ips_translator = {
             'translation-type': 'HDICT',
             'table-name': 'fixed-ips',
@@ -79,7 +90,8 @@ class TestDatasourceDriver(base.TestCase):
             'selector-type': 'DICT_SELECTOR',
             'in-list': True,
             'field-translators':
-                ({'fieldname': 'ip_address', 'translator': self.val_trans},
+                ({'fieldname': 'ip_address',
+                  'translator': datasource_utils.typed_value_trans(Type2)},
                  {'fieldname': 'subnet_id', 'translator': self.val_trans})}
 
         ports_translator = {
@@ -87,7 +99,8 @@ class TestDatasourceDriver(base.TestCase):
             'table-name': 'ports',
             'selector-type': 'DICT_SELECTOR',
             'field-translators':
-                ({'fieldname': 'id', 'translator': self.val_trans},
+                ({'fieldname': 'id',
+                  'translator': datasource_utils.typed_value_trans(Type2)},
                  {'fieldname': 'fixed_ips',
                   'translator': ports_fixed_ips_translator})}
 
@@ -509,7 +522,7 @@ class TestDatasourceDriver(base.TestCase):
                       'translator': self.val_trans}
         rows, k = datasource_driver.DataSourceDriver.convert_obj(resp,
                                                                  translator)
-        self.assertTrue(rows is None)
+        self.assertIsNone(rows)
 
     def test_convert_list_with_id(self):
         # Test a single LIST with an id_column
@@ -520,14 +533,14 @@ class TestDatasourceDriver(base.TestCase):
         rows, k = datasource_driver.DataSourceDriver.convert_obj(resp,
                                                                  translator)
 
-        k1 = self.compute_hash((1, 'a', 'b', 'True'))
+        k1 = self.compute_hash((1, 'a', 'b', True))
 
         self.assertEqual(4, len(rows))
         self.assertEqual(k1, k)
         self.assertIn(('testtable', (k, 1)), rows)
         self.assertIn(('testtable', (k, 'a')), rows)
         self.assertIn(('testtable', (k, 'b')), rows)
-        self.assertIn(('testtable', (k, 'True')), rows)
+        self.assertIn(('testtable', (k, True)), rows)
 
     def test_convert_list_with_id_function(self):
         # Test a single LIST with an id function
@@ -545,7 +558,7 @@ class TestDatasourceDriver(base.TestCase):
         self.assertIn(('testtable', (k, 1)), rows)
         self.assertIn(('testtable', (k, 'a')), rows)
         self.assertIn(('testtable', (k, 'b')), rows)
-        self.assertIn(('testtable', (k, 'True')), rows)
+        self.assertIn(('testtable', (k, True)), rows)
 
     def test_convert_list_without_id(self):
         # Test a single LIST without an id_column
@@ -560,7 +573,7 @@ class TestDatasourceDriver(base.TestCase):
         self.assertIn(('testtable', (1,)), rows)
         self.assertIn(('testtable', ('a',)), rows)
         self.assertIn(('testtable', ('b',)), rows)
-        self.assertIn(('testtable', ('True',)), rows)
+        self.assertIn(('testtable', (True,)), rows)
 
     def test_convert_list_with_sublist(self):
         # Test a single LIST with an id_column
@@ -576,7 +589,7 @@ class TestDatasourceDriver(base.TestCase):
 
         k1 = self.compute_hash((1, 2, 3))
         k2 = self.compute_hash(('a', 'b', 'c'))
-        k3 = self.compute_hash(('True', 'False'))
+        k3 = self.compute_hash((True, False))
         k = self.compute_hash((k1, k2, k3))
 
         self.assertEqual(11, len(rows))
@@ -587,8 +600,8 @@ class TestDatasourceDriver(base.TestCase):
         self.assertIn(('subtable', (k2, 'a')), rows)
         self.assertIn(('subtable', (k2, 'b')), rows)
         self.assertIn(('subtable', (k2, 'c')), rows)
-        self.assertIn(('subtable', (k3, 'True')), rows)
-        self.assertIn(('subtable', (k3, 'False')), rows)
+        self.assertIn(('subtable', (k3, True)), rows)
+        self.assertIn(('subtable', (k3, False)), rows)
         self.assertIn(('testtable', (k, k1)), rows)
         self.assertIn(('testtable', (k, k2)), rows)
         self.assertIn(('testtable', (k, k3)), rows)
@@ -627,7 +640,7 @@ class TestDatasourceDriver(base.TestCase):
 
         self.assertEqual(1, len(rows))
         self.assertIsNone(k)
-        self.assertEqual([('testtable', ('FOO', 'None'))], rows)
+        self.assertEqual([('testtable', ('FOO', None))], rows)
 
     def test_convert_recursive_hdict_single_fields_default_col(self):
         # Test simple fields inside of an HDICT using the default col name.
@@ -1093,14 +1106,18 @@ class TestDatasourceDriver(base.TestCase):
                                     'field-translators': (
                                         {'fieldname': 'a',
                                          'col': 'a1',
-                                         'translator': self.val_trans},
+                                         'translator':
+                                             datasource_utils.
+                                             typed_value_trans(
+                                                 data_types.Bool)},
                                         {'fieldname': 'b',
                                          'col': 'b1',
                                          'translator': self.val_trans})}},
                     {'fieldname': 'testfield2',
                      'translator': {'translation-type': 'HDICT',
                                     'table-name': 'subtable2',
-                                    'id-col': 'id2',
+                                    'parent-key': 'zparent_col3',
+                                    'parent-col-name': 'id2',
                                     'field-translators': (
                                         {'fieldname': 'c',
                                          'col': 'c1',
@@ -1109,7 +1126,8 @@ class TestDatasourceDriver(base.TestCase):
                                          'col': 'd1',
                                          'translator': self.val_trans})}},
                     {'fieldname': 'ztestfield3', 'col': 'zparent_col3',
-                     'translator': self.val_trans},
+                     'translator': datasource_utils.typed_value_trans(
+                         data_types.Str)},
                     {'fieldname': 'testfield4', 'col': 'parent_col4',
                      'translator': {'translation-type': 'VALUE',
                                     'extract-fn': lambda x: x.id}},
@@ -1143,9 +1161,9 @@ class TestDatasourceDriver(base.TestCase):
         self.assertEqual(7, len(schema))
 
         self.assertEqual(({'name': 'id1', 'desc': None},
-                          {'name': 'a1', 'desc': None},
+                          {'name': 'a1', 'desc': None, 'type': 'Bool'},
                           {'name': 'b1', 'desc': None}), schema['subtable1'])
-        self.assertEqual(({'name': 'id2', 'desc': None},
+        self.assertEqual(({'name': 'id2', 'desc': None, 'type': 'Str'},
                           {'name': 'c1', 'desc': None},
                           {'name': 'd1', 'desc': None}), schema['subtable2'])
         self.assertEqual(('id3', 'key3', 'value3'), schema['subtable3'])
@@ -1157,8 +1175,7 @@ class TestDatasourceDriver(base.TestCase):
                            'desc': None},), schema['subtable6'])
         self.assertEqual(
             ({'name': 'parent_col1', 'desc': None},
-             {'name': 'testfield2', 'desc': None},
-             {'name': 'zparent_col3', 'desc': None},
+             {'name': 'zparent_col3', 'desc': None, 'type': 'Str'},
              {'name': 'parent_col4', 'desc': None},
              {'name': 'parent_col5', 'desc': None},
              {'name': 'parent_col6', 'desc': None},
@@ -1191,7 +1208,7 @@ class TestDatasourceDriver(base.TestCase):
         class TestDriver(datasource_driver.DataSourceDriver):
             subtranslator = {'translation-type': 'LIST',
                              'table-name': 'subtable',
-                             'parent-key': 'id', 'val-col': 'val',
+                             'parent-key': 'id_col', 'val-col': 'val',
                              'translator': self.val_trans}
 
             translator = {'translation-type': 'HDICT',
@@ -1218,6 +1235,36 @@ class TestDatasourceDriver(base.TestCase):
             ({'name': 'parent_key', 'desc': None},
              {'name': 'val', 'desc': None}), schema['subtable'])
 
+    def test_get_schema_with_hdict_parent_and_id_col_in_subtranslator(self):
+        class TestDriver(datasource_driver.DataSourceDriver):
+            subtranslator = {'translation-type': 'LIST',
+                             'table-name': 'subtable',
+                             'id-col': 'id', 'val-col': 'val',
+                             'translator': self.val_trans}
+
+            translator = {'translation-type': 'HDICT',
+                          'table-name': 'testtable',
+                          'selector-type': 'DICT_SELECTOR',
+                          'field-translators': ({'fieldname': 'unique_key',
+                                                 'translator': self.val_trans},
+                                                {'fieldname': 'sublist',
+                                                 'translator': subtranslator})}
+
+            TRANSLATORS = [translator]
+
+            def __init__(self):
+                super(TestDriver, self).__init__('', None)
+
+        schema = TestDriver().get_schema()
+
+        self.assertEqual(2, len(schema))
+        self.assertEqual(
+            ({'desc': None, 'name': 'unique_key'},
+             {'desc': None, 'name': 'sublist'}), schema['testtable'])
+        self.assertEqual(
+            ({'desc': None, 'name': 'id'},
+             {'desc': None, 'name': 'val'}), schema['subtable'])
+
     def test_get_schema_with_hdict_id_function(self):
         class TestDriver(datasource_driver.DataSourceDriver):
             translator = {
@@ -1745,7 +1792,7 @@ class TestPollingDataSourceDriver(base.TestCase):
     @mock.patch.object(eventlet.greenthread, 'kill')
     @mock.patch.object(eventlet, 'spawn')
     def test_cleanup(self, mock_spawn, mock_kill):
-        dummy_thread = dict()
+        dummy_thread = mock.MagicMock()
         mock_spawn.return_value = dummy_thread
 
         test_driver = TestPollingDataSourceDriver.TestDriver()
@@ -1762,7 +1809,7 @@ class TestPollingDataSourceDriver(base.TestCase):
         args = {'lazy_tables': ['fake_table']}
         test_driver = fake_datasource.FakeDataSource(args=args)
 
-        self.assertTrue('fake_table' not in test_driver._table_deps)
+        self.assertNotIn('fake_table', test_driver._table_deps)
         test_driver.update_from_datasource()
         self.assertEqual(test_driver.update_number, 0)
 
@@ -1844,7 +1891,7 @@ class TestPushedDriver(base.SqlTestCase):
             {'id': 1, 'name': 'column1', 'status': 'up'},
             {'id': 2, 'name': 'column2', 'status': 'down'}
             ]
-        test_driver.update_entire_data('test_translator', obj)
+        test_driver.replace_entire_table_data('test_translator', obj)
         expected_state = set([
             (1, 'column1', 'up'),
             (2, 'column2', 'down')])
@@ -1864,7 +1911,7 @@ class TestPushedDriver(base.SqlTestCase):
         # test no persist if not enabled
         test_driver = TestPushedDriver.TestDriver(
             args={'ds_id': ds_id, 'persist_data': False})
-        test_driver.update_entire_data('test_translator', obj)
+        test_driver.replace_entire_table_data('test_translator', obj)
         expected_state = set([
             (1, 'column1', 'up'),
             (2, 'column2', 'down')])
@@ -1875,7 +1922,7 @@ class TestPushedDriver(base.SqlTestCase):
         # test data persisted in DB
         test_driver = TestPushedDriver.TestDriver(
             args={'ds_id': ds_id, 'persist_data': True})
-        test_driver.update_entire_data('test_translator', obj)
+        test_driver.replace_entire_table_data('test_translator', obj)
         expected_state = set([
             (1, 'column1', 'up'),
             (2, 'column2', 'down')])
@@ -1899,10 +1946,24 @@ class TestPushedDriver(base.SqlTestCase):
 
 
 class TestExecutionDriver(base.TestCase):
+    class ExtendedExecutionDriver(datasource_driver.ExecutionDriver):
+        """Subclass of test target.
+
+        Execution Driver is an add-on class for datasource driver so
+        it's assumed to have heartbeat_callbacks variable defined in
+        DataService class.
+        """
+        def __init__(self):
+            # A variable defined in datasource_driver
+            self.name = 'TestExecutionDriver'
+            self.heartbeat_callbacks = {}
+            super(TestExecutionDriver.ExtendedExecutionDriver, self).__init__()
+            self.method_structured_args = {
+                'action_struct': {'named': frozenset(['arg2', 'arg3'])}}
 
     def setUp(self):
         super(TestExecutionDriver, self).setUp()
-        self.exec_driver = datasource_driver.ExecutionDriver()
+        self.exec_driver = TestExecutionDriver.ExtendedExecutionDriver()
 
     def test_get_method_nested(self):
         class server(object):
@@ -1941,6 +2002,27 @@ class TestExecutionDriver(base.TestCase):
         # the api
         self.exec_driver._execute_api(nova_client, "action", arg)
 
+    def test_execute_api_structured_args(self):
+        class NovaClient(object):
+            def action_struct(self, arg1, arg2, arg3):
+                return (arg1, arg2, arg3)
+
+        nova_client = NovaClient()
+        arg = {"positional": ["value1", '{"value2":2}'],
+               "named": {"arg3": '{"value3":3}'}}
+        # it will raise exception if the method _execute_api failed to location
+        # the api
+        self.assertEqual(
+            ("value1", {"value2": 2}, {"value3": 3}),
+            self.exec_driver._execute_api(nova_client, "action_struct", arg))
+
+        arg = {"positional": ["value1", "}{invalid"],
+               "named": {"arg3": '{"value3":3}'}}
+
+        self.assertRaisesRegexp(
+            exception.CongressException, '.*invalid format.*',
+            self.exec_driver._execute_api, nova_client, "action_struct", arg)
+
     def test_get_actions_order_by_name(self):
         mock_methods = {'funcA': mock.MagicMock(),
                         'funcH': mock.MagicMock(),
@@ -1985,3 +2067,34 @@ class TestExecutionDriver(base.TestCase):
         api_prefix = 'congress.tests.datasources.test_datasource_driver'
         self.exec_driver.add_executable_client_methods(nova_client, api_prefix)
         self.assertEqual(expected_methods, self.exec_driver.executable_methods)
+
+    @mock.patch.object(eventlet, 'spawn')
+    def test_request_execute_with_wait(self, mock_spawn):
+        thread = mock.MagicMock()
+        mock_spawn.return_value = thread
+
+        context = {'node_id': 'fake_node'}
+        action = 'test-action'
+        action_args = ()
+
+        test_driver = fake_datasource.FakeDataSource()
+        test_driver.request_execute(context, action, action_args, True)
+        mock_spawn.assert_called_once_with(test_driver.execute,
+                                           action, action_args)
+        thread.wait.assert_called_once()
+
+    @mock.patch.object(eventlet, 'spawn')
+    @mock.patch.object(eventlet.greenthread.GreenThread, 'wait')
+    def test_request_execute_without_wait(self, mock_wait, mock_spawn):
+        mock_wait.return_value = mock.MagicMock()
+        mock_spawn.return_value = mock.MagicMock()
+
+        context = {'node_id': 'fake_node'}
+        action = 'test-action'
+        action_args = []
+
+        test_driver = fake_datasource.FakeDataSource()
+        test_driver.request_execute(context, action, action_args, False)
+        mock_spawn.assert_called_once_with(test_driver.execute,
+                                           action, action_args)
+        mock_wait.assert_not_called()
