@@ -81,7 +81,7 @@ def generate_messaging_config():
     mc_fixture = conffixture.ConfFixture(cfg.CONF)
     mc_fixture.conf.transport_url = 'kombu+memory://'
     messaging_config = mc_fixture.conf
-    messaging_config.rpc_response_timeout = 1
+    messaging_config.rpc_response_timeout = 10
     return messaging_config
 
 
@@ -130,10 +130,12 @@ def api_module_path():
     return path
 
 
-def test_path():
-    """Return path to root of top-level tests."""
+def test_path(file=None):
+    """Return path to root of top-level tests. Joined with file if provided."""
     path = source_path()
     path = os.path.join(path, "tests")
+    if file is not None:
+        path = os.path.join(path, file)
     return path
 
 
@@ -420,6 +422,24 @@ def retry_til_exception(expected_exception, f):
         raise TestFailureException("Wrong exception thrown: %s" % e)
 
 
+@tenacity.retry(stop=tenacity.stop_after_attempt(20),
+                wait=tenacity.wait_fixed(1))
+def retry_check_function_return_value_table(f, expected_values):
+    """Check if function f returns expected table."""
+    result = f()
+    actual = set(tuple(x) for x in result)
+    correct = set(tuple(x) for x in expected_values)
+    extra = actual - correct
+    missing = correct - actual
+    if len(extra) > 0 or len(missing) > 0:
+        s = "Actual: %s\nExpected: %s\n" % (result, expected_values)
+        if len(extra) > 0:
+            s += "Extra: %s\n" % extra
+        if len(missing) > 0:
+            s += "Missing: %s\n" % missing
+        raise TestFailureException(s)
+
+
 class FakeRequest(object):
     def __init__(self, body):
         self.body = json.dumps(body)
@@ -438,3 +458,65 @@ class TestFailureException(Exception):
     """
     def __init__(self, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
+
+
+def supported_drivers():
+    """Get list of supported drivers by congress"""
+
+    results = [
+        {"id": "monasca",
+         "description": "Datasource driver that interfaces with monasca."},
+        {"id": "plexxi",
+         "description": "Datasource driver that interfaces with PlexxiCore."},
+        {"id": "doctor",
+         "description": "Datasource driver that allows external systems "
+                        "to push data in accordance with OPNFV Doctor "
+                        "Inspector southbound interface specification."},
+        {"id": "aodh",
+         "description": "Datasource driver that interfaces with aodh."},
+        {"id": "neutronv2_qos",
+         "description": "Datasource driver that interfaces with QoS "
+                        "extension of OpenStack Networking aka Neutron."},
+        {"id": "cloudfoundryv2",
+         "description": "Datasource driver that interfaces with cloudfoundry"},
+        {"id": "heat",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "orchestration aka heat."},
+        {"id": "nova",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "Compute aka nova."},
+        {"id": "murano",
+         "description": "Datasource driver that interfaces with murano"},
+        {"id": "neutronv2",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "Networking aka Neutron."},
+        {"id": "swift",
+         "description": "Datasource driver that interfaces with swift."},
+        {"id": "ironic",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "bare metal aka ironic."},
+        {"id": "cinder",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "cinder."},
+        {"id": "fake_datasource",
+         "description": "This is a fake driver used for testing"},
+        {"id": "config",
+         "description": "Datasource driver that allows OS configs retrieval."},
+        {"id": "glancev2",
+         "description": "Datasource driver that interfaces with OpenStack "
+                        "Images aka Glance."},
+        {"id": "vcenter",
+         "description": "Datasource driver that interfaces with vcenter"},
+        {"id": "keystonev3",
+         "description": "Datasource driver that interfaces with keystone."},
+        {"id": "keystone",
+         "description": "Datasource driver that interfaces with keystone."},
+        {"id": "mistral",
+         "description": "Datasource driver that interfaces with Mistral."},
+        {"id": "vitrage",
+         "description": "Datasource driver that accepts Vitrage "
+                        "webhook alarm notifications."},
+        {"id": "monasca_webhook",
+         "description": "Datasource driver that accepts Monasca webhook "
+                        "alarm notifications."}]
+    return results
